@@ -1,14 +1,14 @@
-package IO::Pipe::Producer;
+## Producer.pm
+##
+## Author:  Robert W. Leach
+## Date:    3/7/2003
+## Company: Los Alamos National Laboratory
 
-use 5.012003;
-use strict;
-use warnings;
+package IO::Pipe::Producer;
+use base qw(IO::Pipe);
 use Carp;
 
-our @ISA = qw(IO::Pipe);
-use base qw(IO::Pipe);
-
-our $VERSION = '1.10';
+$Producer::VERSION = '1.5';
 
 #NOTICE
 #
@@ -32,6 +32,7 @@ our $VERSION = '1.10';
 #from LANL.
 
 
+
 #Constructor
 
 sub new
@@ -81,7 +82,7 @@ sub getSubroutineProducer
       {
 	$error = "ERROR:Producer.pm:getSubroutineProducer:A referenced " .
 	  "subroutine is required as the first argument to " .
-	    "getSubroutineProducer.";
+	    "getSubroutineProducer.\n";
 	$Producer::errstr = $error;
 	carp($error);
 	return(undef);
@@ -89,8 +90,7 @@ sub getSubroutineProducer
 
     #Create a pipe
     my $stdout_pipe = $self->SUPER::new();
-    my($stderr_pipe);
-    $stderr_pipe = $self->SUPER::new() if(wantarray);
+    my $stderr_pipe = $self->SUPER::new();
 
     #Fork off the Producer
     if(defined($pid = fork()))
@@ -103,8 +103,7 @@ sub getSubroutineProducer
 
 	    #Create a read file handle
 	    $stdout_pipe->reader();
-	    $stderr_pipe->reader() if(wantarray);
-
+	    $stderr_pipe->reader();
 	    #Return the read file handle to the consumer
 	    if(wantarray)
 	      {return(bless($stdout_pipe,ref($self)),
@@ -120,8 +119,8 @@ sub getSubroutineProducer
 	    #Create a write file handle for the Producer
 	    $stdout_pipe->writer();
 	    $stdout_pipe->autoflush;
-	    $stderr_pipe->writer()  if(defined($stderr_pipe));
-	    $stderr_pipe->autoflush if(defined($stderr_pipe));
+	    $stderr_pipe->writer();
+	    $stderr_pipe->autoflush;
 
 	    #Redirect standard outputs to the pipes or kill the child
 	    if(!open(STDOUT,">&",\${$stdout_pipe}))
@@ -133,7 +132,7 @@ sub getSubroutineProducer
 		$Producer::errstr = $error;
 		croak($error);
 	      }
-	    elsif(defined($stderr_pipe) && !open(STDERR,">&",\${$stderr_pipe}))
+	    elsif(!open(STDERR,">&",\${$stderr_pipe}))
 	      {
 		$error = "ERROR:Producer.pm:getSubroutineProducer:Can't " .
 		  "redirect stderr to pipe: [" .
@@ -146,9 +145,9 @@ sub getSubroutineProducer
 	    #Call the subroutine passed in (ignore it's return value)
 	    $producer_sub->(@params);
 
-	    #Close the writer pipes
-	    close($stdout_pipe);
-	    close($stderr_pipe) if(defined($stderr_pipe));
+	    #Close the pipes (hence it's unnecessary to do so in the consumer)
+	    $stdout_pipe->close();
+	    $stderr_pipe->close();
 
 	    #Successfully exiting the child process
 	    exit(0);
@@ -156,7 +155,8 @@ sub getSubroutineProducer
       }
     else
       {
-	$error = "ERROR:Producer.pm:getSubroutineProducer:fork() didn't work!";
+	$error = "ERROR:Producer.pm:getSubroutineProducer:fork() didn't " .
+	  "work!\n";
 	$Producer::errstr = $error;
 	carp($error);
 	return(undef);
@@ -171,90 +171,101 @@ sub getSystemProducer
   }
 
 
-1;
-__END__
 
 =head1 NAME
 
-IO::Pipe::Producer - Perl extension for IO::Pipe
-
-=head1 SYNOPSIS
- 
-  #Example 1 (Call a subroutine & grab its standard output):
- 
-  use IO::Pipe::Producer;
-  $obj = new IO::Pipe::Producer();
-  $stdout_file_handle =
-    $obj->getSubroutineProducer(\&mysub,
-                                @mysub_params);
-  while(<$stdout_file_handle>)
-    {print}
-  close($stdout_file_handle);
- 
-  #Example 2 (Call a subroutine & grab its standard output & standard error):
- 
-  ($stdout_file_handle,$stderr_file_handle) =
-    $obj->getSubroutineProducer(\&mysub,
-                                @mysub_params);
- 
-  #It is recommended to use IO::Select when reading more than 1 file handle:
-  use IO::Select;
-  my $sel = new IO::Select;
-  $sel->add($stdout_file_handle,$stderr_file_handle);
-  while(my @fhs = $sel->can_read())
-    {
-      foreach my $fh (@fhs)
-        {
-          my $line = <$fh>;
-          unless(defined($line))
-            {
-              $sel->remove($fh);
-              close($fh);
-              next;
-            }
-          if($fh == $stdout_file_handle)
-            {$messages .= $line}
-          elsif($fh == $stderr)
-            {$errors .= $line}
-        }
-     }
- 
-  #Example 3 (Grab the standard output & standard error of a system call):
- 
-  use IO::Pipe::Producer;
-  $obj = new IO::Pipe::Producer();
-  ($stdout_fh,$stderr_fh) =
-    $obj->getSystemProducer("echo \"Hello World!\"");
-
-
-=head1 ABSTRACT
-
-Producer.pm is useful for chaining large data processing subroutines or system calls.  Instead of making each call serially and waiting for a return, you can create a Producer that will continuously generate output that can be immediately processed.  You can even split up input and run subroutines in parallel.  Producer.pm is basically a way to pipe the standard output of a forked subroutine or system call to a file handle in your parent process.
-
-=head1 DESCRIPTION
-
-Producer.pm is a module that provides methods to fork off a subroutine or system call and return handles on the standard output (STDOUT and STDERR).  If you have (for example) a subroutine that processes a very large text file and performs a task on each line, but you need to perform further processing, normally you would have to wait until the subroutine returns to get its output.  If the subroutine prints its output to STDOUT (and STDERR) or you can edit it to do so, you can call it using a Producer so that you can use the returned handle to continuously process each line as it's "produced".  You can chain subroutines together like this by having your subroutine itself create a Producer.  This is similar to using open() to run a system call, except that with this module, you can get a handle on STDERR and use it with subroutines as well.  And by dividing up your input, you can take advantage of multi-core systems and process your data in a parallel fashion.
-
-Note that the handles retuned are open file handles.  It is your job to close them once you are finished with them.
-
-=head1 NOTES
-
-This module was originally written as a simple subroutine that used IO::Pipe.  It adds one method and a helper method (Note: The getSystemProducer method calls getSubroutineProducer).  It functions by opening STDOUT/STDERR as input, which is the basic definition of a pipe.  Those input file handles are what is returned.
-
-=head1 BUGS
-
-No known bugs.  Please report them to I<E<lt>rleach@princeton.eduE<gt>> if you find any.
-
-=head1 SEE ALSO
-
-L<IO::Pipe>
-L<IO::Select>
+IO::Pipe::Producer
 
 =head1 AUTHOR
 
-Robert William Leach, E<lt>rleach@princeton.eduE<gt>
+IO::Pipe::Producer was written by Robert W. Leach I<E<lt>robleach@lanl.govE<gt>> in 2005.
 
-=head1 COPYRIGHT AND LICENSE
+=head1 SYNOPSIS
+ 
+ # Module which provides 2 methods: getSubroutineProducer
+ # and getSystemProducer.  They take a subroutine reference
+ # (with associated arguments) and a system call
+ # respectively and return (blessed) handles on their
+ # streaming standard output and standard error output.
+ 
+ 
+ # EXAMPLES of usage
+ 
+ use IO::Pipe::Producer;
+ $obj = new IO::Pipe::Producer();
+ $stdout_fh =
+   $obj->getSubroutineProducer($subroutine_reference,
+                               @subroutine_parameters);
+ 
+ # OR
+ 
+ use IO::Pipe::Producer;
+ $obj = new IO::Pipe::Producer();
+ ($stdout_fh,$stderr_fh) =
+   $obj->getSubroutineProducer($subroutine_reference,
+                               @subroutine_parameters);
+ 
+ # OR
+ 
+ use IO::Pipe::Producer;
+ $stdout_fh = new IO::Pipe::Producer($subroutine_reference,
+ 				    @subroutine_parameters);
+ 
+ # OR
+ 
+ use IO::Pipe::Producer;
+ ($stdout_fh,$stderr_fh) =
+   new IO::Pipe::Producer($subroutine_reference,
+                          @subroutine_parameters);
+ 
+ # Then you can read the returned handles like any other
+ # file handle...
+ 
+ while(<$stdout_fh>)
+   {print "STDOUT From Producer: $_"}
+ while(<$stderr_fh>)
+   {print "STDERR From Producer: $_"}
+ 
+ # You can also do the same thing with system calls using
+ # the getSystemProducer subroutine.  However, this feature
+ # is not accessible via the new constructor
+ 
+ use IO::Pipe::Producer;
+ $obj = new IO::Pipe::Producer();
+ $stdout_fh =
+   $obj->getSystemProducer("echo \"Hello World!\"");
+ 
+ use IO::Pipe::Producer;
+ $obj = new IO::Pipe::Producer();
+ ($stdout_fh,$stderr_fh) =
+   $obj->getSystemProducer("echo \"Hello World!\"");
+ 
+ # However, this is exactly the same as:
+ 
+ use IO::Pipe::Producer;
+ $stdout_fh = new Producer(sub{system(@_)},
+			   "echo \"Hello World!\"");
+ 
+ # OR
+ 
+ use IO::Pipe::Producer;
+ ($stdout_fh,$stderr_fh) =
+   new IO::Pipe::Producer(sub{system(@_)},
+			  "echo \"Hello World!\"");
+ 
+=head1 ABSTRACT
+
+Producer.pm is useful for piggy-backing large data processing subroutines or system calls.  Instead of making each call serially and waiting for a return or playing with temporary files, you can create a Producer that will continuosly generate output that can be further processed right away in your script.  One benefit is immediate feedback.  It's basically a way to pipe the standard output of a forked subroutine or system call to a file handle in your parent process.
+
+=head1 DESCRIPTION
+
+Producer.pm is a module that provides methods to fork off a subroutine or system call and return handles on the standard output (STDOUT and STDERR).  If you have (for example) a subroutine that processes a very large text file and performs a task on each line, but you need to perform further processing with either other subroutines or in main, normally you would have to wait until the subroutine returns to get its output (either by returning it or opening a temporary file that it produced) and continue processing.  If the subroutine prints its output to STDOUT (and STDERR) or you can edit it to do so, you can call it using a Producer so that you can use the returned handle to continuously process each line as it's "produced".  You can chain subroutines together like this by having your subroutine itself create a Producer.  This is similar to using open() to run a system call, except that with this module, you can get a handle on STDERR and use it with subroutines as well.
+
+=head1 NOTES
+
+This module was originally written as a simple subrotuine in a library that simply used IO::Pipe.  I decided to make it into a subclass of IO::Pipe even though it only really adds one method and a helper method (Note: The getSystemProducer method calls getSubroutineProducer) because libraries seem antiquated to me.  I'm open to better design suggestions however.  I also decided to bless the returns of all the methods because they were retuning IO::Pipe objects anyway.  The basic trick this module uses can be gleaned from the IO::Pipe documentation if you read between the lines and combine various tidbits from different sections, but to simplify it, it simply "opens" the file handle inside the IO::Pipe data structure to accept output from the output file handles (STDOUT/STDERR) as input, which is the basic definition of a pipe.
+
+=head1 NOTICE
 
 This software and ancillary information (herein called "SOFTWARE") called Producer.pm is made available under the terms described here.  The SOFTWARE has been approved for release with associated LA-CC number LA-CC-05-060.
 
@@ -262,5 +273,16 @@ Unless otherwise indicated, this software has been authored by an employee or em
 
 If SOFTWARE is modified to produce derivative works, such modified SOFTWARE should be clearly marked, so as not to confuse it with the version available from LANL.
 
+=head1 BUGS
+
+No known bugs.  Please report them to I<E<lt>robleach@lanl.govE<gt>> if you find any.
+
+=head1 SEE ALSO
+
+L<IO::Pipe>
 
 =cut
+
+
+
+1;
